package com.googlecode.wicketslides;

import java.util.ArrayList;
import java.util.List;

import org.apache.wicket.ResourceReference;
import org.apache.wicket.behavior.HeaderContributor;
import org.apache.wicket.markup.html.IHeaderContributor;
import org.apache.wicket.markup.html.IHeaderResponse;
import org.apache.wicket.markup.html.list.ListItem;
import org.apache.wicket.markup.html.list.ListView;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.markup.html.resources.JavascriptResourceReference;

public class SlidesPanel extends Panel {

	private List imageList;
	private List thumbList;
	private Boolean thumbs;
	private Integer width;
	private Integer height;
	private Integer thumbWidth;
	private Integer thumbHeight;
	private Boolean timed;
	private Integer fadeDuration;
	private Integer delay;
	private Boolean showArrow;
	private Boolean embedLinks;
	private Boolean showInfoPane;

	protected static final ResourceReference MOOTOOLS_1_11 = new JavascriptResourceReference(
			SlidesPanel.class, "smoothgallery/scripts/mootools.v1.11.js");
	protected static final ResourceReference MOOTOOLS_1_11_EXT = new JavascriptResourceReference(
			SlidesPanel.class, "smoothgallery/scripts/mootools.v1.11.ext.js");
	protected static final ResourceReference SMOOTHGALLERY = new JavascriptResourceReference(
			SlidesPanel.class, "smoothgallery/scripts/jd.gallery.js");
	protected static final ResourceReference SMOOTHGALLERY_SET = new JavascriptResourceReference(
			SlidesPanel.class, "smoothgallery/scripts/jd.gallery.set.js");
	protected static final ResourceReference SMOOTHGALLERY_TRANSITIONS = new JavascriptResourceReference(
			SlidesPanel.class,
			"smoothgallery/scripts/jd.gallery.transitions.js");
	private static final ResourceReference SMOOTHGALLERY_HISTORY_MANAGER = new JavascriptResourceReference(
			SlidesPanel.class, "smoothgallery/scripts/HistoryManager.js");
	protected static final ResourceReference SMOOTHGALLERY_CSS = new ResourceReference(
			SlidesPanel.class, "smoothgallery/css/jd.gallery.css");
	protected static final ResourceReference SMOOTHGALLERY_CSS_LAYOUT = new ResourceReference(
			SlidesPanel.class, "smoothgallery/css/layout.css");

	private SlidesPanel(String id) {
		super(id);
	}

	private void processPanel() {

		add(HeaderContributor.forCss(SMOOTHGALLERY_CSS_LAYOUT));
		add(HeaderContributor.forCss(SMOOTHGALLERY_CSS));
		add(HeaderContributor.forJavaScript(MOOTOOLS_1_11));
		add(HeaderContributor.forJavaScript(MOOTOOLS_1_11_EXT));
		add(HeaderContributor.forJavaScript(SMOOTHGALLERY));
		add(HeaderContributor.forJavaScript(SMOOTHGALLERY_SET));
		add(HeaderContributor.forJavaScript(SMOOTHGALLERY_TRANSITIONS));
		add(HeaderContributor.forJavaScript(SMOOTHGALLERY_HISTORY_MANAGER));
		add(new HeaderContributor(new IHeaderContributor() {
			@Override
			public void renderHead(IHeaderResponse response) {
				response.renderString("<style type=\"text/css\">"
						+ "#myGallery" + "{	width: " + width + "px; height:"
						+ height + "px; z-index:5; border: 0; } " + "</style>");
			}
		}));
		add(new HeaderContributor(new IHeaderContributor() {

			@Override
			public void renderHead(IHeaderResponse response) {
				response.renderOnDomReadyJavascript(""
						+ "var myGallery = new gallery($('myGallery'), {"
						+ "timed: "
						+ timed
						+ ", "
						+ "fadeDuration: "
						+ fadeDuration
						+ ", "
						+ "delay: "
						+ delay
						+ ", "
						+ "useHistoryManager: true,"
						+ "thumbWidth: "
						+ thumbWidth
						+ ", "
						+ "thumbHeight: "
						+ thumbHeight
						+ ", "
						+ "showInfopane: "
						+ showInfoPane
						+ ", "
						+ "embedLinks: "
						+ embedLinks
						+ ", "
						+ "thumbCloseCarousel: true, "
						+ "showCarouselLabel: true, "
						+ "textShowCarousel: 'Pictures', "
						+ "showArrows: "
						+ showArrow
						+ ", "
						+ "showCarousel: "
						+ thumbs
						+ " "
						+ "});"
						+ (thumbs ? "myGallery.showCarousel();" : "")
						+ "HistoryManager.start();");
			}

		}));

		ListView listview = new ListView("imageElement", imageList) {

			protected void populateItem(ListItem item) {
				loader.populate(item);
			}

		};

		add(listview);

	}

	private ImageLoader loader;

	public static class Builder {
		private String id;
		private List imageList = new ArrayList(1);
		private List thumbList = new ArrayList(1);
		private Boolean thumbs = false;
		private Integer width = 480;
		private Integer height = 360;
		private Integer thumbWidth = 48;
		private Integer thumbHeight = 36;
		private Boolean timed = false;
		private Integer fadeDuration = 600;
		private Integer delay = 6000;
		private Boolean showArrow = true;
		private Boolean embedLinks = true;
		private Boolean showInfoPane = true;
		private ImageLoader loader = new ImageLoader();

		public Builder(String id) {
			this.id = id;
		}

		public SlidesPanel build() {
			SlidesPanel panel = new SlidesPanel(id);
			panel.imageList = imageList;
			panel.thumbList = thumbList;
			panel.thumbs = thumbs;
			panel.width = width;
			panel.height = height;
			panel.thumbWidth = thumbWidth;
			panel.thumbHeight = thumbHeight;
			panel.timed = timed;
			panel.fadeDuration = fadeDuration;
			panel.delay = delay;
			panel.showArrow = showArrow;
			panel.embedLinks = embedLinks;
			panel.showInfoPane = showInfoPane;
			panel.loader = loader;
			if (thumbs == null) {
				if (thumbList.contains(null)
						|| imageList.size() != thumbList.size()) {
					panel.thumbs = false;
				} else if (imageList.size() == thumbList.size()) {
					panel.thumbs = true;
				}
			}

			panel.processPanel();
			return panel;
		}

		public Builder showThumbs(Boolean showThumbs) {
			if (showThumbs == null)
				this.thumbs = false;
			this.thumbs = showThumbs;
			return this;
		}

		public Builder thumbList(List<String> thumbList) {
			if (thumbList == null || thumbList.isEmpty())
				this.thumbs = false;
			this.thumbList = thumbList;
			return this;
		}

		public Builder size(Integer width, Integer height) {
			if (width == null || height == null) {
				throw new IllegalArgumentException(
						"Either provide both width and height, or don't call size() at all");
			}
			this.width = width;
			this.height = height;
			return this;
		}

		public Builder thumbSize(Integer width, Integer height) {
			if (width == null || height == null) {
				throw new IllegalArgumentException(
						"Either provide both width and height, or don't call size() at all");
			}
			this.thumbWidth = width;
			this.thumbHeight = height;
			return this;
		}

		public Builder timed(Boolean timed) {
			this.timed = timed;
			return this;
		}

		public Builder showArrow(Boolean showArrow) {
			this.showArrow = showArrow;
			return this;
		}

		public Builder fadeDuration(Integer fadeDuration) {
			this.fadeDuration = fadeDuration;
			return this;
		}

		public Builder delay(Integer delay) {
			this.delay = delay;
			return this;
		}

		public Builder addImage(String imageUrl, String thumbUrl) {
			this.imageList.add(imageUrl);
			this.thumbList.add(thumbUrl);
			return this;
		}

		public Builder addImage(String imageUrl) {
			this.imageList.add(imageUrl);
			return this;
		}

		public Builder addImageList(List<String> listImagesUrl) {
			this.imageList = listImagesUrl;
			return this;
		}

		public Builder addThumbList(List<String> listThumbsUrl) {
			this.thumbList = listThumbsUrl;
			return this;
		}

		public Builder loadImageWith(ImageLoader loader) {
			this.loader = loader;
			return this;
		}

		public void addImage(SlideshowImage mainImage, SlideshowImage thumbImag2) {
			this.imageList.add(mainImage);
			this.thumbList.add(thumbImag2);
		}

	}

}
